<?php

namespace Drupal\dynamic_links\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\dynamic_links\DynamicLinkInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Returns responses for dynamic links.
 */
class DynamicLinksController implements ContainerInjectionInterface {

  /**
   * Constructs the object.
   *
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   The current user.
   */
  public function __construct(
    protected AccountInterface $currentUser,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
    );
  }

  /**
   * Builds the redirect response for the dynamic link.
   *
   * @param \Drupal\dynamic_links\DynamicLinkInterface $dynamic_link
   *   The target dynamic link.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   The redirect response.
   */
  public function redirect(DynamicLinkInterface $dynamic_link): RedirectResponse {
    $url = $dynamic_link->getFirstAvailable($this->currentUser)['url']->toString();
    return new RedirectResponse($url);
  }

  /**
   * Checks access to the dynamic link.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The account to check access for.
   * @param \Drupal\dynamic_links\DynamicLinkInterface $dynamic_link
   *   The target dynamic link.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(AccountInterface $account, DynamicLinkInterface $dynamic_link): AccessResultInterface {
    $result = $dynamic_link->getFirstAvailable($account);
    return ($result['url'] ? AccessResult::allowed() : AccessResult::forbidden())->addCacheableDependency($result['cacheability']);
  }

}
